@@ -1,4 +1,7 @@
-#include <boost/shared_ptr.hpp>
+#ifndef _TIMESTAMP_H_
+#define _TIMESTAMP_H_
+
+#include <memory>
 
 // Impl abstract class; implemented by platform-specific classes
 class TimestampImpl {
@@ -7,12 +10,13 @@ public:
   virtual bool future() const = 0;
   virtual bool less(const TimestampImpl* other) const = 0;
   virtual bool greater(const TimestampImpl* other) const = 0;
+  virtual double diff_secs(const TimestampImpl* other) const = 0;
 };
 
 class Timestamp {
 private:
-  boost::shared_ptr<const TimestampImpl> p_impl;
-  
+  std::shared_ptr<const TimestampImpl> p_impl;
+
 public:
   Timestamp();
   Timestamp(double secs);
@@ -21,7 +25,7 @@ public:
   bool future() const {
     return p_impl->future();
   }
-  
+
   // Comparison operators
   bool operator<(const Timestamp& other) const {
     return p_impl->less(other.p_impl.get());
@@ -29,4 +33,11 @@ public:
   bool operator>(const Timestamp& other) const {
     return p_impl->greater(other.p_impl.get());
   }
+
+  // Diff
+  double diff_secs(const Timestamp& other) const {
+    return p_impl->diff_secs(other.p_impl.get());
+  }
 };
+
+#endif // _TIMESTAMP_H_
